# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
path.py

Plugs into the routine module to allow for the easy construction of paths and
path following.
'''
import mud, mudsys, room, movement, random, gainlevel



################################################################################
# functions
################################################################################
def leads_to(frm, to):
    '''returns whether from leads directly to to'''
    for ex in frm.exnames:
        if frm.exit(ex).dest == to:
            return True
    return False

def shortest_path_bfs(frm, to, ignore_doors = False, stay_zone = True,
                      ignore = None):
    '''calculates the shortest path, but uses a breadth first search. More
       efficient than depth-first seach for very short paths with lots of
       branches or very large muds.
    '''
    if frm == to:
        return [ frm ]

    depth = [ [ frm ] ]

    if ignore == None:
        ignore = set()
    ignore.add(frm)

    # figure out what zone we're doing this from
    zone = None
    if stay_zone:
        zone = frm.locale

    # keep going until we find to, or we can't go any deeper
    found = False
    while not found:
        layer = [ ]
        for room in depth[-1]:
            for dir in room.exnames:
                dest = room.exit(dir).dest
                if dest == None or dest in ignore:
                    pass
                elif zone != None and dest.locale != zone:
                    pass
                elif dest == to:
                    found = True
                    layer = [ ]
                    break
                else:
                    layer.append(dest)
                ignore.add(dest)
            if found == True:
                break
        if len(layer) > 0:
            depth.append(layer)
        if found == True or len(layer) == 0:
            break

    # no path found
    if found == False:
        return None

    # find the rooms that link each other, in reverse order
    path  = [ to ]
    order = range(len(depth))
    order.reverse()
    for i in order:
        for room in depth[i]:
            if leads_to(room, path[-1]):
                path.append(room)
                break

    path.reverse()
    return path

def shortest_path_dfs(frm, to, ignore_doors = False, stay_zone = True,
                      ignore = None):
    '''returns the steps needed to take to go from one room to another. More
       efficient than breadth-first search for very long paths with only a few
       branches, or very small muds.'''
    path = []

    if ignore == None:
        ignore = set()

    # a list of rooms we ignore for tracking. Add ourself so we don't loop back
    ignore.add(frm)

    # if we're the target room, return an empty list
    if frm == to:
        return [frm]

    zone = None
    if stay_zone:
        zone = frm.locale

    # build the shortest path 
    for ex in frm.exnames:
        # check if it has a door, and if we ignore it
        if frm.exit(ex).is_closed:
            continue

        # get the dest room. if there is none, skip this exit
        next_room = frm.exit(ex).dest
        
        if next_room == None:
            continue

        # if we already know this is a dead end or a loopback, skip it
        if (next_room in ignore or
            (stay_zone and not next_room.locale == zone)):
            continue

        next_path = shortest_path(next_room, to, ignore_doors, stay_zone,ignore)

        # dead end
        if len(next_path) == 0:
            continue

        # we found a path, append ourself
        next_path.insert(0, frm)

        # are we shorter than the previous one?
        if len(path) == 0 or len(path) > len(next_path):
            path = next_path

    return path

# set whether we are using bfs or dfs as our main pathing method
shortest_path = shortest_path_bfs

def path_to_dirs(path):
    '''takes a path of rooms and converts it to directions'''

    dirs = []
    i    = 0
    while i < len(path) - 1:
        for ex in path[i].exnames:
            if path[i].exit(ex).dest == path[i+1]:
                dirs.append(ex)
                break
        i = i + 1

    # return the directions we generated, if any
    return dirs

def build_patrol(rms, reverse = True, ignore_doors = False, stay_zone = True, asdirs = True):
    '''builds a set of directions that need to be followed to do a patrol
       between the rooms. If reverse is true, also supplies the directions
       to loop back on itself'''
    if reverse == True:
        loopback = [x for x in rms]
        loopback.reverse()
        rms      = rms + loopback[1:]

    path = []
    i    = 0
    while i < len(rms) - 1:
        path.extend(shortest_path(rms[i], rms[i+1], ignore_doors, stay_zone))
        i += 1
    if asdirs:
        return path_to_dirs(path)
    else:
        return path

def step(frm, to, ignore_doors = False, stay_zone = True):
    '''returns the first step needed to take to go from one room to another'''
    steps = shortest_path(frm, to, ignore_doors, stay_zone)
    if steps == None or len(steps) <= 1:
        return None
    return path_to_dirs(steps)[0]



################################################################################
# commands
################################################################################
def cmd_path(ch, cmd, arg):
    '''Usage: path <room>

       Prints out a Python list of the directions needed to move from your
       current location to a specified destination.'''
    try:
        dest, = mud.parse_args(ch, True, cmd, arg, "room")
    except: return

    path = build_patrol([ch.room, dest])

    if len(path) == 0:
        ch.send("Path doesn't exist")
    else:
        ch.send(str(path))

def cmd_runto(ch, cmd, arg):
    '''Usage: biegnij [do] [waypoint]

    Run to selected waypoint. If no waypoint selected, show available waypoints
    in current zone.'''
    waypoints = {'matherion' : {'alchemik': ['sklep5', 'Mikstury i zioła leczące'], 'arena': ['arenawrota', 'Walcz z potworami na arenie'], 'bank' : ['bank1', 'Zdeponuj/pobierz pieniądze'], 'biblioteka' : ['biblioteka1', 'Przeczytaj twórczość innych osób'], 'fontanna' : ['fontanna', 'Główny punkt Matherionu'], 'katedra' : ['katedrawejscie', 'Miejsce modlitw'], 'krawiec' : ['sklep6', 'Ubrania na każdą okazję'], 'płatnerz' : ['sklep3', 'Zbroje i tarcze'], 'ratusz' : ['ratusz1', 'Władze miasta'], 'sklep podróżnika' : ['sklep4', 'Ekwipunek niezbędny w podróży'], 'spożywczy' : ['sklep1', 'Jedzenie oraz napoje'], 'tawerna' : ['tawerna1', 'Odpoczynek przy muzyce i trunkach'], 'zbrojmistrz' : ['sklep2', 'Wszelaka broń'], 'jubiler' : ['sklep8', 'Pierścienie i naszyjniki']}}
    if len(arg) == 0:
        if not ch.room.locale in waypoints:
            ch.send("Nie ma punktów docelowych w tej strefie.")
            return
        buf = ['Możliwe miejsca podróży:']
        for waypoint in sorted(waypoints[ch.room.locale]):
            buf.append(waypoint.capitalize() + ": " + waypoints[ch.room.locale][waypoint][1])
        ch.send('\r\n'.join(buf))
        return
    try:
        name, = mud.parse_args(ch, True, cmd, arg, "[do] string(punkt)")
    except:
        return
    name = name.lower()
    if not name in waypoints[ch.room.locale]:
        ch.send("Nie ma takiego punktu docelowego.")
        return
    dest, = mud.parse_args(ch, True, cmd, waypoints[ch.room.locale][name][0], "room")
    path = build_patrol([ch.room, dest], False)
    for direction in path:
        ex, success = movement.try_move(ch, direction, True)
        if not success:
            break

def cmd_track(ch, cmd, arg):
    '''Usage: trop <cel>

    Try find selected character or NPC in current zone.'''
    #first, we try find target in world
    try:
        vict, = mud.parse_args(ch, True, cmd, arg, 'ch.world.noself')
    except:
        return
    if vict.room.locale != ch.room.locale:
        ch.send('Nie ma tutaj śladów ' + vict.name  +  '$g.')
        return
    if 'tropić istoty' in ch.aux('cooldowns').names:
        ch.send('Przed chwilą próbowałeś coś tropić. Odpocznij trochę zanim zrobisz to ponownie.')
        return
    mud.message(ch, None, None, None, True, 'to_room', '$n rozgląda się po okolicy.')
    #check if player can find target
    hasSkill = -1
    chroll = ch.aux('stats').values[2] + random.randint(0, 99)
    terrain = {'Inside' : 10, 'City' : 10, 'Road' : 10, 'Alley' : 10, 'Bridge' : 10,
               'Cavern' : 10, 'Ice' : 10, 'Glacier' : 10, 'Mountains' : 7, 'Deep Swamps' : 7,
               'Deep Forest' : 5, 'Hills' : 5, 'High Hills' : 5, 'Swamp' : 5, 'Desert' : 5,
               'Field' : 1, 'Plains' : 1, 'Meadow' : 1, 'Forest' : 1, 'Sand' : 1}
    if 'Tropienie' in ch.aux('skills').names:
        hasSkill = ch.aux('skills').names.index('Tropienie')
        chroll += ch.aux('skills').levels[hasSkill]
    rooms = build_patrol([ch.room, vict.room], False, True, True, False)
    dirs = build_patrol([ch.room, vict.room], False)
    chance = vict.hidden
    for rm in rooms:
        if not rm.terrain in terrain.keys():
            chance += 100
        else:
            chance += terrain[rm.terrain]
    if chroll >= chance:
        dir_name = ["północ", "wschód", "południe", "zachód", "północny-wschód", "północny-zachód",
                 "południowy-zachód", "południowy-wschód", "góra", "dół"]
        dir_count = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
        for i in range(0, len(dir_name) - 1):
            dir_count[i] = dirs.count(dir_name[i])
        index = dir_count.index(max(dir_count))
        ch.send('Twoim zdaniem, ' + vict.name + ' powinnien znajdować się na ' + dir_name[index] + ' stąd.')
        gain = max(chance / 10, 1)
    else:
        gain = 1
        ch.send('Nie ma tutaj śladów ' + vict.name  +  '$g.')
    if hasSkill > -1:
        ch.aux('skills').exp[hasSkill] += gain
        gainlevel.check_exp(ch, 'Tropienie', 'skill')
    else:
        ch.addskill('Tropienie', gain)
    ch.aux('stats').secondvalues[2] += gain
    gainlevel.check_exp(ch, 'int', 'stat')
    ch.aux('cooldowns').names.append('tropić istoty')
    ch.aux('cooldowns').times.append(10)

################################################################################
# initialization
################################################################################

# add our commands
mudsys.add_cmd("path", None, cmd_path, "admin", False)
mudsys.add_cmd("biegnij", None, cmd_runto, "player", False)
mudsys.add_cmd("trop", None, cmd_track, "player", False)

# mud initialization
mud.build_patrol = build_patrol
