# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
rest.py

Recovering health points and fatigue during time and bed item type for faster
recovering.
'''

import mudsys, mud, char, hooks, event, math, olc, auxiliary, mudsock, storage, utils

################################################################################
# item type definition
################################################################################

class BedData:
    '''Data for bed.'''

    __item_type__ = 'bed'

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.bonus = 100

        if set != None:
            self.bonus = set.readInt('bonus')

    def copyTo(self, to):
        to.bonus = self.bonus

    def copy(self):
        newData = BedData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeInt('bonus', self.bonus)

################################################################################
# BED OLC
################################################################################
def bedolc_menu(sock, data):
    '''display our menu interface.'''
    buf = []
    buf.append('{g1) Premia do regeneracji  : {c%s%%' % str(data.bonus))
    sock.send('\r\n'.join(buf))

def bedolc_chooser(sock, data, option):
    '''figure out what we want to edit, and display a prompt.'''
    if option != '1':
        return olc.MENU_CHOICE_INVALID
    sock.send("Wprowadź premię do regeneracji (w procentach): ")
    return 1

def bedolc_parser(sock, data, choice, arg):
    '''parse out what we want to change a specified field to.'''
    try:
        arg = int(arg)
    except:
        return False

    if choice == 1:
        data.bonus = max(int(arg), 1)
    else:
        return False
    return True

def bedolc_to_proto(data):
    '''return a script that will generate relevant info to create this item.'''
    buf = 'me.get_type_data("bed").bonus = ' + str(data.bonus) + '\n'
    return buf

################################################################################
# game commands
################################################################################
def cmd_immortal(ch, cmd, arg):
    '''Command set or unset immortal flag on wizard.'''
    if ch.hasPrefs('immortal'):
        if ch.prfs == 'immortal':
            ch.prfs = 'none'
        else:
            prfs = ch.prfs.split(', ')
            prfs.remove('immortal')
            ch.prfs = ', '.join(prfs)
        event.start_event(ch, 180, condition_event, None, '')
        ch.send('Przestajesz być nieśmiertelny.')
    else:
        if ch.prfs == 'none':
            ch.prfs = 'immortal'
        else:
            ch.prfs += ', immortal'
        ch.send('Od teraz jesteś nieśmiertelny.')

################################################################################
# Events
################################################################################
def resting_event(owner, void, info):
    '''Function recover health points and fatigure over time'''
    if owner == None:
        return
    #count restore multiplier - base on player position, thirst, hunger and furniture (if any)
    if owner.position == "standing" or owner.position == "flying":
        multiplier = 0.1
    elif owner.position == "sitting":
        multiplier = 0.15
    elif owner.position == "sleeping" or owner.position == "unconscious":
        multiplier = 0.2
    if owner.on != None:
        if owner.on.istype('bed'):
            multiplier += ((multiplier / 100) * owner.on.get_type_data('bed').bonus)
    if owner.is_pc:
        condition_mod = float(owner.aux('stats').values[6]) / float(owner.aux('stats').secondvalues[6])
        multiplier -= (multiplier * condition_mod)
        condition_mod = float(owner.aux('stats').values[7]) / float(owner.aux('stats').secondvalues[7])
        multiplier -= (multiplier * condition_mod)
    #raise hit points
    cur_val = owner.aux('stats').values[4]
    max_val = owner.aux('stats').secondvalues[4]
    recover = math.ceil(float(max_val) * multiplier)
    if recover < 1:
        recover = 1
    cur_val = cur_val + recover
    if owner.position == "unconscious":
        owner.position = "sitting"
    if cur_val > max_val:
        cur_val = max_val
    owner.aux('stats').values[4] = int(cur_val)
    #lower fatigue for players only
    if owner.is_pc:
        cur_val = owner.aux('stats').values[5]
        max_val = owner.aux('stats').secondvalues[5]
        recover = math.ceil(float(max_val) * multiplier)
        if recover < 1:
            recover = 1
        cur_val = cur_val - recover
        if cur_val < 0:
            cur_val = 0
        owner.aux('stats').values[5] = int(cur_val)
    event.start_event(owner, 70, resting_event, None, '')

def condition_event(owner, void, info):
    '''Event raise hunger and thirst of character'''
    if owner == None:
        return
    if owner.hasPrefs('immortal'):
        return
    #raise hunger
    cur_val = owner.aux('stats').values[6]
    max_val = owner.aux('stats').secondvalues[6]
    if cur_val < max_val:
        cur_val = cur_val + 1
    owner.aux('stats').values[6] = int(cur_val)
    percent = (float(cur_val) / float(max_val)) * 100
    if percent > 74:
        owner.send("Jesteś głodny!")
    #raise thirst
    cur_val = owner.aux('stats').values[7]
    max_val = owner.aux('stats').secondvalues[7]
    if cur_val < max_val:
        cur_val = cur_val + 1
    owner.aux('stats').values[7] = int(cur_val)
    percent = (float(cur_val) / float(max_val)) * 100
    if percent > 74:
        owner.send("Jesteś spragniony!")
    #start event again
    event.start_event(owner, 180, condition_event, None, '')

################################################################################
# hooks
################################################################################
def resting_hook(info):
    '''start resting over time'''
    ch, = hooks.parse_info(info)
    if ch == None or not ch.is_pc:
        return
    event.start_event(ch, 70, resting_event, None, '')

def condition_hook(info):
    '''raise hunger and thirst over time'''
    ch, = hooks.parse_info(info)
    if ch == None or not ch.is_pc:
        return
    if not ch.hasPrefs('immortal'):
        event.start_event(ch, 180, condition_event, None, '')

################################################################################
# initialization and unloading
################################################################################

#initialize our item data and its OLC
mudsys.item_add_type(BedData.__item_type__, BedData)
olc.item_add_olc(BedData.__item_type__,
                 bedolc_menu, bedolc_chooser, bedolc_parser,
                 None, bedolc_to_proto)

mudsys.add_cmd("niesmiertelnosc", None, cmd_immortal, "wizard", True)

hooks.add("char_to_game", resting_hook)
hooks.add("char_to_game", condition_hook)

def __unload__():
    '''things that need to be detached when the module is un/reloaded'''
    hooks.remove("char_to_game", resting_hook)
    hooks.remove("char_to_game", condition_hook)
